#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
# encoding: utf-8

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.conf import settings
from django.contrib.sites.models import Site

from datetime import datetime, time, date, timedelta

from ...models import Attestation
from .... import __version__

import sys

class Command(BaseCommand):

    help = 'Send email warning for attestations about to expire'

    def handle(self, *args, **options):
        for expiration_reminder in settings.EXPIRATION_REMINDERS:
            expiration_date = date.today() + timedelta(days=expiration_reminder)
            attestations_about_to_expire = Attestation.objects.filter(locked=True, expiration_date__range=(datetime.combine(expiration_date, time.min),
                                                                                                           datetime.combine(expiration_date, time.max)))
            if attestations_about_to_expire:
                current_site = Site.objects.get_current()
                template_path = 'attestations/attestation_about_to_expire_email.txt'
                for attestation in attestations_about_to_expire:
                    subject = "Attestation for experiment %s is about to expire" % \
                        attestation.experiment.fullname()

                    send_mail(subject,
                              render_to_string(template_path,
                                               {
                                                 'attestation': attestation,
                                                 'beat_version': __version__,
                                                 'site': current_site,
                                                 }
                                               ),
                              settings.DEFAULT_FROM_EMAIL,
                              [attestation.experiment.author.email])
                self.stdout.write('{} attestation(s) about to expire'.format(attestations_about_to_expire.count()))
            else:
                self.stdout.write('No attestation(s) about to expire in {} day(s)'.format(expiration_reminder))

        call_command('clean_attestations', '--noinput')
