#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

from actstream.managers import ActionManager, stream
from actstream.registry import check
from beat.web.team.models import Team

class BeatActionManager(ActionManager):

    @stream
    def public_for_user(self, obj, **kwargs):
        """
        Only return public actions for given user
        """
        check(obj)
        kwargs['public'] = True
        return self.any(obj, **kwargs)

    @stream
    def for_user(self, obj, **kwargs):
        """
        Stream of most recent actions where given user or team he
        owns is part of is the target as well as all other public
        activities.
        Keyword arguments will be passed to Action.objects.filter
        """
        check(obj)
        ctype = ContentType.objects.get_for_model(obj)
        query = Q(
                    target_content_type=ctype,
                    target_object_id=obj.pk,
                )

        if obj.teams.count() > 0:
            team_ctype = ContentType.objects.get_for_model(Team)
            team_ids = [team.id for team in obj.teams.all()]
            query |= Q(
                        target_content_type=team_ctype,
                        target_object_id__in=team_ids,
                    )

        public_query = ~Q(
                    actor_content_type=ctype,
                    actor_object_id=obj.pk,
                ) & ~Q(
                    target_content_type=ctype,
                    target_object_id=obj.pk,
                ) & ~Q(
                    action_object_content_type=ctype,
                    action_object_object_id=obj.pk,
                ) & Q(
                        public=True
                    )

        return self.filter(query | public_query, **kwargs)
