#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import os
import shutil

from django.test import TestCase
from django.conf import settings

from urlparse import urlparse

import simplejson as json

from .models import Shareable


def tearDownModule():

    if os.path.exists(settings.CACHE_ROOT):
        shutil.rmtree(settings.CACHE_ROOT)
    if os.path.exists(settings.PREFIX):
        shutil.rmtree(settings.PREFIX)


class BaseTestCase(TestCase):
    class Meta:
        model = None


    def checkObjectDBSharingPreferences(self, contribution, reference):
        if not(reference.has_key('status')):
            reference['status'] = 'private'

        if not(reference.has_key('shared_with')):
            reference['shared_with'] = []

        if not(reference.has_key('shared_with_team')):
            reference['shared_with_team'] = []

        if not(reference.has_key('usable_by')):
            reference['usable_by'] = []

        if not(reference.has_key('usable_by_team')):
            reference['usable_by_team'] = []

        if reference['status'] == 'public':
            self.assertEqual(contribution.sharing, Shareable.PUBLIC)
        elif reference['status'] == 'usable':
            self.assertEqual(contribution.sharing, Shareable.USABLE)
        elif reference['status'] == 'shared':
            self.assertEqual(contribution.sharing, Shareable.SHARED)
        else:
            self.assertEqual(contribution.sharing, Shareable.PRIVATE)

        self.assertEqual(contribution.shared_with.count(), len(reference['shared_with']))
        self.assertEqual(contribution.shared_with_team.count(), len(reference['shared_with_team']))

        for user in contribution.shared_with.iterator():
            self.assertTrue(user.username in reference['shared_with'])

        for team in contribution.shared_with_team.iterator():
            self.assertTrue(team.fullname() in reference['shared_with_team'])


        if hasattr(contribution, 'usable_by'):
            self.assertEqual(contribution.usable_by.count(), len(reference['usable_by']))
            for user in contribution.usable_by.iterator():
                self.assertTrue(user.username in reference['usable_by'])

        if hasattr(contribution, 'usable_by_team'):
            self.assertEqual(contribution.usable_by_team.count(), len(reference['usable_by_team']))
            for team in contribution.usable_by_team.iterator():
                self.assertTrue(team.fullname() in reference['usable_by_team'])


    def checkDBSharingPreferences(self, username, object_name, reference):
        db_object = self.Meta.model.objects.get(author__username=username, name=object_name, version=1)
        self.checkObjectDBSharingPreferences(db_object, reference)


    def checkResponse(self, response, status_code, location=None, content_type=None):
        self.assertEqual(response.status_code, status_code)

        if location is not None:
            parsed_location = urlparse(response.get('Location', ''))
            self.assertTrue(response.has_header('Location'))
            self.assertEqual(parsed_location.path, location)

        if content_type is not None:
            self.assertTrue(response.has_header('Content-Type'))
            self.assertEqual(response.get('Content-Type', ''), content_type)
            self.assertNotEqual(len(response.content), 0)

            if content_type == 'application/json':
                return json.loads(response.content)
            else:
                return response.content
        else:
            standard_codes = [400, 401, 403, 404, 405, 415]
            if status_code not in standard_codes:
                self.assertEqual(len(response.content), 0)
