# -*- coding: utf-8 -*-

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from __future__ import unicode_literals

from django.db import migrations, models

from ...common.models import get_declaration

from ..models import validate_database

import logging
logger = logging.getLogger(__name__)


def refresh_databases(apps, schema_editor):
    '''Refreshes each database so datasets/outputs are recreated'''

    Database = apps.get_model("databases", "Database")
    DatabaseSetOutput = apps.get_model("databases", "DatabaseSetOutput")

    Database.declaration = property(get_declaration)
    Database.fullname = lambda self: '%s/%d' % (self.name, self.version)

    if Database.objects.count(): print('')

    for db in Database.objects.order_by('id'):
        print("Refreshing protocols for database `%s'..." % db.fullname())
        core = validate_database(db.declaration)
        core.name = db.fullname()
        for proto in db.protocols.all():
            for set in proto.sets.all():
                for output in set.template.outputs.all():
                    try:
                        DatabaseSetOutput(template=output, set=set,
                            hash=core.hash_output(proto.name, set.name,
                              output.name)).save()
                    except KeyError:
                        logger.warn('Database output %s/%d.%s.%s.%s does ' \
                            'not exist' % (db.name, db.version, proto.name,
                              set.name, output.name))
                        continue


class Migration(migrations.Migration):

    dependencies = [
        ('dataformats', '0001_initial'),
        ('databases', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel('DatabaseOutput', 'DatabaseSetTemplateOutput'),
        migrations.AlterUniqueTogether(
            name='databasesettemplateoutput',
            unique_together=set([('template', 'name')]),
        ),
        migrations.CreateModel(
            name='DatabaseSetOutput',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('hash', models.CharField(unique=True, max_length=64)),
                ('set', models.ForeignKey(related_name='outputs', to='databases.DatabaseSet')),
                ('template', models.ForeignKey(related_name='instances', to='databases.DatabaseSetTemplateOutput')),
            ],
        ),
        migrations.RunPython(refresh_databases),
    ]
