#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext

from .models import Database


def view(request, name, version=None):
    """Displays a single database"""

    # Retrieve the database
    try:
        if version is not None:
            version = int(version)

            database = Database.objects.for_user(request.user, True).get(name__iexact=name,
                                            version=version)
        else:
            database = Database.objects.for_user(request.user, True).filter(name__iexact=name).order_by('-version')[0]
    except:
        raise Http404

    # Render the page
    return render_to_response('databases/view.html',
                              {'database': database,
                              },
                              context_instance=RequestContext(request))


def ls(request):
    '''List all accessible databases to the request user'''

    # orders databases so that the latest information is displayed first
    objects = Database.objects.for_user(request.user, True).order_by('-creation_date')
    objects = Database.filter_latest_versions(objects)

    return render_to_response('databases/list.html',
            dict(
                objects=objects,
                author=request.user,
                owner=True,
                ),
            context_instance=RequestContext(request),
            )
