#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings

import simplejson
import beat.core.experiment
from ...common import storage


def set_block_order(apps, schema_editor):
    '''Set block order for existing experiments'''

    Experiment = apps.get_model("experiments", "Experiment")
    Block = apps.get_model("experiments", "Block")

    total = Experiment.objects.count()
    for i, e in enumerate(Experiment.objects.order_by('id')):

        fullname = '%s/%s/%s/%d/%s' % (
            e.author.username,
            e.toolchain.author.username,
            e.toolchain.name,
            e.toolchain.version,
            e.name,
            )

        print("Updating blocks for experiment %d/%d (%s, id=%d)..." % \
            (i+1, total, fullname, e.id))

        xp_decl = simplejson.loads(storage.get_file_content(e,
          'declaration_file'))
        tc_decl = simplejson.loads(storage.get_file_content(e.toolchain,
          'declaration_file'))

        xp = beat.core.experiment.Experiment(settings.PREFIX, (xp_decl,
          tc_decl))

        if xp.errors:
            message = "The experiment `%s' isn't valid (skipping " \
                "block update), due to the following errors:\n  * %s"
            print message % (fullname, '\n * '.join(xp.errors))
            continue

        # Goes, in order, setting block inner order
        for order_0, block_name in enumerate(xp.setup().keys()):

            block = Block.objects.get(experiment=e, name=block_name)
            block.execution_order = order_0 + 1
            block.save()


class Migration(migrations.Migration):

    dependencies = [
        ('experiments', '0005_scheduler_addons_4'),
    ]

    operations = [
        migrations.AddField(
            model_name='block',
            name='execution_order',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterModelOptions(
            name='block',
            options={'ordering': ['experiment_id', 'execution_order']},
        ),
        migrations.RunPython(set_block_order),
    ]
