#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from .models import Library
from .serializers import LibrarySerializer
from .serializers import FullLibrarySerializer
from .serializers import LibraryCreationSerializer

from ..code.api import ShareCodeView, RetrieveUpdateDestroyCodeView
from ..code.serializers import CodeDiffSerializer

from ..common.api import (CheckContributionNameView, ListContributionView,
    ListCreateContributionView)

from ..code.api import DiffView

#----------------------------------------------------------


class CheckLibraryNameView(CheckContributionNameView):
    """
    This view sanitizes a library name and
    checks whether it is already used.
    """
    model = Library


#----------------------------------------------------------


class ShareLibraryView(ShareCodeView):
    """
    This view allows to share a library with
    other users and/or teams
    """
    model = Library


#----------------------------------------------------------


class ListLibrariesView(ListContributionView):
    """
    List all available libraries
    """
    model = Library
    serializer_class = LibrarySerializer


#----------------------------------------------------------


class ListCreateLibrariesView(ListCreateContributionView):
    """
    Read/Write end point that list the libraries available
    from a given author and allows the creation of new libraries
    """
    model = Library
    serializer_class = LibrarySerializer
    writing_serializer_class = LibraryCreationSerializer
    namespace = 'api_libraries'


#----------------------------------------------------------


class RetrieveUpdateDestroyLibrariesView(RetrieveUpdateDestroyCodeView):
    """
    Read/Write/Delete endpoint for a given library
    """
    model = Library
    serializer_class = FullLibrarySerializer


    def do_update(self, request, author_name, object_name, version=None):
        modified, library = super(RetrieveUpdateDestroyLibrariesView, self).do_update(request, author_name, object_name, version)

        if modified:
            # Delete existing experiments using the library (code changed)
            experiments = []
            for item in library.referencing.all():
                for algorithm in item.used_by_algorithms.all():
                    experiments.append(list(set(map(lambda x: x.experiment,
                        algorithm.blocks.iterator()))))
            for experiment in set(experiments): experiment.delete()

        return modified, library


#----------------------------------------------------------


class DiffLibraryView(DiffView):
    """
    This view shows the differences between two libraries
    """
    model = Library
    serializer_class = CodeDiffSerializer
