#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from ..common.serializers import DynamicFieldsSerializer, ContributionSerializer, ContributionCreationSerializer
from .models import Plotter, PlotterParameter, DefaultPlotter
from rest_framework import serializers

from ..code.serializers import CodeSerializer, CodeCreationSerializer
from ..libraries.serializers import LibraryReferenceSerializer
from ..dataformats.serializers import ReferencedDataFormatSerializer


import beat.core.plotter
import simplejson as json

class PlotterSerializer(ContributionSerializer):

    dataformat = serializers.CharField(source="dataformat.fullname")

    class Meta(ContributionSerializer.Meta):
        model = Plotter
        default_fields = [
                'name', 'dataformat',
                ]

class PlotterParameterSerializer(ContributionSerializer):

    class Meta(ContributionSerializer.Meta):
        model = PlotterParameter
        exclude = []
        default_fields = [
                'name',
                ]

class DefaultPlotterSerializer(DynamicFieldsSerializer):

    dataformat = serializers.CharField(source="dataformat.fullname")
    plotter    = serializers.CharField(source="plotter.fullname")
    parameter  = serializers.CharField(source="parameter.fullname")

    class Meta(DynamicFieldsSerializer.Meta):
        model = DefaultPlotter
        default_fields = [
                'dataformat', 'plotter', 'parameter',
                ]


#----------------------------------------------------------


class PlotterCreationSerializer(CodeCreationSerializer):
    class Meta(CodeCreationSerializer.Meta):
        model = Plotter
        beat_core_class = beat.core.plotter


#----------------------------------------------------------


class PlotterAllSerializer(CodeSerializer):
    dataformat              = serializers.SerializerMethodField()
    declaration_file        = serializers.SerializerMethodField()
    description_file        = serializers.SerializerMethodField()
    source_code_file        = serializers.SerializerMethodField()
    referenced_libraries    = LibraryReferenceSerializer(many=True)

    class Meta(CodeSerializer.Meta):
        model = Plotter


#----------------------------------------------------------


class FullPlotterSerializer(PlotterAllSerializer):

    class Meta(PlotterAllSerializer.Meta):
        default_fields = PlotterAllSerializer.Meta.default_fields + PlotterAllSerializer.Meta.extra_fields


#----------------------------------------------------------

class PlotterParameterCreationFailedException(Exception):
    pass

class PlotterParameterCreationSerializer(ContributionCreationSerializer):
    class Meta(ContributionCreationSerializer.Meta):
        model = PlotterParameter
        #beat_core_class = beat.core.PlotterParameter
    def create(self, validated_data):
        plotterparameter = None

        if not validated_data.has_key("name"):
            raise serializers.ValidationError('No name provided')

        try:
            plotterparameter = PlotterParameter.objects.get(author=self.context['request'].user, name=validated_data['name'])
        except:
            pass

        if plotterparameter is not None:
            raise serializers.ValidationError('A plotterparameter with this name already exists')

        validated_data['data'] = {}
        plotterparameter = PlotterParameter.objects.create(**validated_data)
        if plotterparameter is None:
            raise PlotterParameterCreationFailedException()
        return plotterparameter


#----------------------------------------------------------


class PlotterParameterAllSerializer(ContributionSerializer):
    data            = serializers.SerializerMethodField()
    description     = serializers.SerializerMethodField()
    plotter         = serializers.SerializerMethodField()

    class Meta(ContributionSerializer.Meta):
        model = PlotterParameter

    #def get_referencing_experiments(self, obj):
    #    user = self.context.get('user')

    #    experiments = obj.experiments.for_user(user, True).order_by('-creation_date')
    #    serializer = ExperimentSerializer(experiments, many=True)
    #    referencing_experiments = serializer.data

    #    # Put the pending experiments first
    #    ordered_result = filter(lambda x: x['creation_date'] is None, referencing_experiments)
    #    ordered_result += filter(lambda x: x['creation_date'] is not None, referencing_experiments)

    #    return ordered_result

    #def get_new_experiment_url(self, obj):
    #    return obj.get_new_experiment_url()

#----------------------------------------------------------


class FullPlotterParameterSerializer(PlotterParameterAllSerializer):

    class Meta(PlotterParameterAllSerializer.Meta):
        #exclude = ['declaration']
        exclude = []
        #default_fields = PlotterParameterAllSerializer.Meta.default_fields + PlotterParameterAllSerializer.Meta.extra_fields
        default_fields = ['id', 'accessibility', 'modifiable', 'deletable', 'is_owner', 'name', 'fork_of', 'last_version', 'previous_version', 'short_description', 'description', 'version', 'creation_date', 'data', 'plotter']

    def get_data(self, obj):
        return json.loads(obj.data)

    def get_plotter(self, obj):
        if obj.plotter is not None:
            return obj.plotter.fullname()
        else:
            return "undefined plotter"

    #def get_plotter(self, obj):
    #    return obj.author.username

        #"accessibility": "public",
        #"modifiable": true,
        #"deletable": true,
        #"is_owner": false,
        #"name": "plot/bar/1",
        #"fork_of": null,
        #"last_version": true,
        #"previous_version": null,
        #"short_description": "Default parameters for bar plots",
        #"description": "Raw content",
        #"version": 1,
        #"creation_date": "2015-09-03T16:55:47.620000",
