/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */
// this controller handles the 'groups' partial, where groups of experiments are managed
angular.module('reportApp').controller('GroupsController', ['$scope', 'ReportService', function ($scope, ReportService){
	let vm = this;

	// testing stuffs
	vm.test = 'test';
	vm.report = $scope.report;

	const getAnalyzerFromExpName = (expName) => {
		let analyzers = vm.report.all_experiments[expName].declaration.analyzers;
		let analyzer_block = analyzers[Object.keys(analyzers)[0]];

		return analyzer_block.algorithm;
	};

	// ReportService stuff to be used directly in ui
	vm.groups = ReportService.groups;
	vm.removeExperimentFromGroup = ReportService.removeExperimentFromGroup;
	vm.addExperimentToGroup = (expName, groupName) => {
		let analyzer = getAnalyzerFromExpName(expName);
		ReportService.addExperimentToGroup(expName, groupName);
		ReportService.groups.find(g => g.name === groupName).analyzer = analyzer;
	};

	vm.createGroup = function(name) {
		ReportService.createGroup(name);
		vm.newGroupName = '';
	};
	vm.deleteGroup = ReportService.deleteGroup;

	// createGroup input val
	vm.newGroupName = '';
	// report's experiments
	vm.experiments = $scope.report_experiments;
	// same analyzer
	vm.compatibleExperiments = (analyzer) => {
		if(!vm.report.all_experiments){
			return {};
		}

		return Object.entries(vm.report.all_experiments)
		.filter(([eName, e]) => analyzer === '' || getAnalyzerFromExpName(eName) === analyzer)
		.reduce((o, [eName, e]) => Object.assign(o, { [eName]: e }), {});
	};

	// a group panel's classes
	// one panel is the active group ('panel-success')
	vm.classArr = (groupName) => {
		let a = ['panel', groupName === ReportService.activeGroup ? 'panel-success' : 'panel-default'];
		return a;
	};

	// set active group
	vm.setActiveGroup = (groupName) => {
		ReportService.activeGroup = groupName;
	};

	console.log(vm);
}]);
