/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */
//This controller retrieves data from the reportFactory/experimentFactory through the REST API and associates it with the $scope
//The $scope is ultimately bound to the report view
angular.module('reportApp').controller('reportController',['$scope', 'reportFactory', 'experimentFactory', 'plotterFactory', 'dataFactory', '$q', 'ReportService', function ($scope, reportFactory, experimentFactory, plotterFactory, dataFactory, $q, ReportService){
	$scope.q = $q;
	$scope.user;
	$scope.report_id;
	$scope.url_prefix;
	$scope.reportFactory = reportFactory;
	$scope.report = {};
	$scope.report_experiments = {};
	$scope.num_report_items = 0;
	$scope.max_num = 0;
	$scope.item_content = [];
	$scope.table_item_content = [];
	$scope.selectedObject = {};

	$scope.tables_details = {};
	$scope.plots_details = {};
	$scope.report_experiments_blocks = {};
	$scope.report_experiments_blocks_merged_blocks = [];
	$scope.report_experiments_blocks_computation_times = {};

	$scope.report_experiments_alias = {};
	$scope.report_experiments_alias_from_content = {};
	$scope.floating_point_precision = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
	$scope.floating_point_precision.selected = 10;
	$scope.report_algorithm_parameters = {};
	$scope.report_algorithm_parameters_experiment = {};
	$scope.sorted_tables = [];   //tables that were sorted
	$scope.sorted_experiments_keys_tables = {};
	$scope.sorted_experiments_alias_keys_tables = {};
	$scope.sorted_experiments_keys_reverse = {};
	$scope.sorted_experiments_keys_tables_sortkey = {};

	$scope.init = function(user, report_id, url_prefix, data_itemcontent_file, data_table_itemcontent_file){
		$scope.user   = user;
		$scope.report_id = report_id;
		$scope.url_prefix = url_prefix;
		getReportData($scope.user, $scope.report_id);
		$scope.item_content = dataFactory.getData(data_itemcontent_file);
		$scope.table_item_content = dataFactory.getData(data_table_itemcontent_file);
	};

	$scope.initWithReportNumber = function(report_number, url_prefix, data_itemcontent_file, data_table_itemcontent_file){
		$scope.report_number = report_number;
		$scope.url_prefix = url_prefix;
		getReportDataFromNumber($scope.report_number);
		$scope.item_content = dataFactory.getData(data_itemcontent_file);
		$scope.table_item_content = dataFactory.getData(data_table_itemcontent_file);
	};

	// saves a data object to an different object by iterating through
	// the data obj's entries and assigning them one-by-one.
	// Why? JS is pass-by-value, but complex datatypes (Objects, arrays, funcs) are pointers.
	// Angular depends on these pointers in two-way bindings of complex datatypes.
	// If one assigns a different obj, the two-way binding is lost!
	function mutateSave(saveObj, dataObj){
		let t = typeof(dataObj);

		if (t === 'object' && saveObj) {
			Object.entries(dataObj)
			.forEach(([key, val]) => {
				saveObj[key] = val;
			});
		} else if (t === 'array' && typeof(saveObj) === 'array') {
			saveObj.length = 0;
			dataObj.forEach(e => saveObj.push(e));
		}
	}

	// saves the report data from the API to the $scope
	function saveReportData(reportData){
		mutateSave($scope.report, reportData);

		$scope.report.url_prefix = $scope.url_prefix;

		if($scope.report.content.alias_experiments != undefined){
			mutateSave($scope.report_experiments_alias, $scope.report.content.alias_experiments);
		}

		// save groups data to the model
		ReportService.loadGroups($scope.report.content.groups);
	}

	function getReportData(user, report_id){
		reportFactory.getReportInformation(user, report_id, $scope.url_prefix)
		.success(function (reportData){
			saveReportData(reportData);
			let promises = [];

			function lastTask(){
				getPlotters();
			}

			let experiments = $scope.report.experiments;
			if(experiments != undefined){
				promises.push(getExperimentDataForAuthor(experiments));
				$scope.q.all(promises).then(lastTask);
			}
		})
		.error(function (error){
			$scope.status = 'Unable to load report data: ' + error.message;
		});

		$('#tabs_progress').hide();
	}

	function getReportDataFromNumber(report_number, $q){
		reportFactory.getReportInformationFromNumber(report_number, $scope.url_prefix)
		.success(function (reportData){
			saveReportData(reportData);
			let promises = [];

			function lastTask(){
				getPlotters();
			}
			if(experiments != undefined){
				promises.push(getExperimentDataReportNumber(report_number));
				$scope.q.all(promises).then(lastTask);
			}
		})
		.error(function (error){
			$scope.status = 'Unable to load report data: ' + error.message;
		});

		$('#tabs_progress').hide();
	}

	function getExperimentDataReportNumber(report_number){
		experimentFactory.getAllExperimentResults($scope.url_prefix, report_number)
		.then(function (experiments){
			mutateSave($scope.report_experiments, experiments);

			$scope.report.all_experiments = experiments;
			let experiment_aliases = Object.keys(experiments);
			for(let i = 0; i < experiment_aliases.length; i++){
				let experiment_alias = experiment_aliases[i];

				if($scope.report.content.alias_experiments != undefined){
					if($scope.report.content.alias_experiments[experiment_alias] != undefined){
						setExperimentAliasFromContent(experiment_alias, $scope.report.content.alias_experiments[experiment_alias]);
					}
					else{
						setExperimentAlias(experiment_alias);
					}
				}
				else{
					setExperimentAlias(experiment_alias);
				}
				intersectBlocks(experiment_alias);
				algorithmParameters(experiment_alias);
			}
			//sort experiments in case it's sorted
			if(!(jQuery.isEmptyObject($scope.report.content.sorted_tables_experiments)) &&
				!(jQuery.isEmptyObject($scope.report.content.sorted_tables_alias_experiments)) &&
				!(jQuery.isEmptyObject($scope.report.content.sorted_tables_keys_reverse)) &&
				!(jQuery.isEmptyObject($scope.report.content.sorted_tables_sortkey))){
				$scope.sorted_experiments_keys_tables = $scope.report.content.sorted_tables_experiments;
				if($scope.report.status == "locked"){
					//special case for locked reports
					$scope.sorted_experiments_keys_tables = $scope.report.content.sorted_tables_alias_experiments;
				}
				$scope.sorted_experiments_alias_keys_tables = $scope.report.content.sorted_tables_alias_experiments;
				$scope.sorted_experiments_keys_reverse = $scope.report.content.sorted_tables_keys_reverse;
				$scope.sorted_experiments_keys_tables_sortkey = $scope.report.content.sorted_tables_sortkey;
			}
			for(let i = 0; i < Object.keys($scope.sorted_experiments_keys_tables_sortkey).length; i++){
				$scope.sorted_tables.push(Object.keys($scope.sorted_experiments_keys_tables_sortkey)[i]);
			}
		});
	}

	function getExperimentDataForAuthor(all_experiments){
		experimentFactory.getAllExperimentResultsForAuthor($scope.user, $scope.report_id, $scope.url_prefix)
		.then(function (experiments){
			mutateSave($scope.report_experiments, experiments);

			$scope.report.all_experiments = experiments;
			let experiment_aliases = Object.keys(experiments);
			for(let i = 0; i < experiment_aliases.length; i++){
				let experiment_alias = experiment_aliases[i];

				if($scope.report.content.alias_experiments != undefined){
					if($scope.report.content.alias_experiments[experiment_alias] != undefined){
						setExperimentAliasFromContent(experiment_alias, $scope.report.content.alias_experiments[experiment_alias]);
					}
					else{
						setExperimentAlias(experiment_alias);
					}
				}
				else{
					setExperimentAlias(experiment_alias);
				}
				intersectBlocks(experiment_alias);
				algorithmParameters(experiment_alias);
			}

			//sort experiments in case it's sorted
			if(!(jQuery.isEmptyObject($scope.report.content.sorted_tables_experiments)) &&
				!(jQuery.isEmptyObject($scope.report.content.sorted_tables_alias_experiments)) &&
				!(jQuery.isEmptyObject($scope.report.content.sorted_tables_keys_reverse)) &&
				!(jQuery.isEmptyObject($scope.report.content.sorted_tables_sortkey))){
				$scope.sorted_experiments_keys_tables = $scope.report.content.sorted_tables_experiments;
				if($scope.report.status == "locked"){
					//special case for locked reports
					$scope.sorted_experiments_keys_tables = $scope.report.content.sorted_tables_alias_experiments;
				}
				$scope.sorted_experiments_alias_keys_tables = $scope.report.content.sorted_tables_alias_experiments;
				$scope.sorted_experiments_keys_reverse = $scope.report.content.sorted_tables_keys_reverse;
				$scope.sorted_experiments_keys_tables_sortkey = $scope.report.content.sorted_tables_sortkey;
			}
			for(let i = 0; i < Object.keys($scope.sorted_experiments_keys_tables_sortkey).length; i++){
				$scope.sorted_tables.push(Object.keys($scope.sorted_experiments_keys_tables_sortkey)[i]);
			}

		});
	}

	function getExperimentData(experiment_id){
		experimentFactory.getExperimentInformation($scope.url_prefix, experiment_id)
		.then(function (experimentData){
			let experiments = $scope.report_experiments;
			experiments[experiment_id] = experimentData;
			mutateSave($scope.report_experiments, experiments);
			$scope.report.all_experiments = experiments;

			if($scope.report.content.alias_experiments != undefined){
				if($scope.report.content.alias_experiments[experiment_id] != undefined){
					setExperimentAliasFromContent(experiment_id, $scope.report.content.alias_experiments[experiment_id]);
				}
				else{
					setExperimentAlias(experiment_id);
				}
			}
			else{
				setExperimentAlias(experiment_id);
			}

			intersectBlocks(experiment_id);
			algorithmParameters(experiment_id);
		});
	}

	function getPlotters(){
		plotterFactory.getPlotters($scope.url_prefix)
		.success(function (plottersData){
			$scope.report.plotters = plottersData;
			getPlotterParameterData();
		})
		.error(function (error){
			$scope.status = 'Unable to load plotters data - ' + ' : ' + error.message;
		});
	}

	function getDefaultPlotters(){
		plotterFactory.getDefaultPlotters($scope.url_prefix)
		.success(function (defaultPlottersData){
			$scope.report.defaultplotters = defaultPlottersData;
			checkContent();
		})
		.error(function (error){
			$scope.status = 'Unable to load default plotters parameter data - ' + ' : ' + error.message;
		});
	}

	function getPlotterParameterData(){
		plotterFactory.getPlotterParameter($scope.url_prefix)
		.success(function (plotterParameterData){
			$scope.report.plotterparameter = plotterParameterData;
			//$scope.report.plotterparameter.push({"name":"Default"});
			getDefaultPlotters();
		})
		.error(function (error){
			$scope.status = 'Unable to load plotter parameter data - ' + ' : ' + error.message;
		});
	}

	function checkContent(){
		if($scope.report.content != undefined){

			let orderkeys = [];
			angular.forEach($scope.report.content, function(value, key){
				orderkeys.push(key);
			});

			orderkeys.sort(function(a,b){
				a = a.split("_");
				b = b.split("_");
				a_num = parseInt(a[a.length-1]) || -1;
				b_num = parseInt(b[b.length-1]) || -1;

				return a_num - b_num;
			});
			$scope.report.orderedcontent = orderkeys;

			if($scope.report.content.floating_point_precision != undefined){
				$scope.floating_point_precision.selected = $scope.report.content.floating_point_precision;
			}

			for(let i = 0; i < $scope.report.orderedcontent.length; i++){
				let id_content = $scope.report.orderedcontent[i];
				addElementToReport(id_content);
			}
		}
	}

	function addElementToReport(id_content){
		let type = id_content.split("_")[0];
		$scope.$broadcast("addSavedElement", id_content, type);
	}

	function intersectBlocks(experiment_id){
		$scope.report_experiments_blocks[experiment_id] = Object.keys($scope.report_experiments[experiment_id].blocks_status);
		let values = $.map($scope.report_experiments_blocks[experiment_id], function(value, key) { return value; });
		$scope.report_experiments_blocks_merged_blocks.push(values);
	}

	function algorithmParameters(experiment_id){
		//go through globals
		angular.forEach($scope.report_experiments[experiment_id].declaration.globals, function(value_algorithm, key_algorithm){
			//only get algorithms
			if(key_algorithm.indexOf("/") != -1){
				$scope.report_algorithm_parameters[key_algorithm] = value_algorithm;
				let experiment_algorithm_parameter = {};

				if($scope.report_algorithm_parameters_experiment[experiment_id] == undefined){
					$scope.report_algorithm_parameters_experiment[experiment_id] = {};
				}

				if($scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm] == undefined){
					$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm] = {};
				}

				//go through each blocks of the experiment and save parameters for given algorithm
				angular.forEach($scope.report_experiments[experiment_id].declaration.blocks, function(value_block, key_block){
					//check blocks that matches globals
					if($scope.report_experiments[experiment_id].declaration.blocks[key_block]['algorithm'] == key_algorithm){
						if($scope.report_experiments[experiment_id].declaration.blocks[key_block]['parameters'] != undefined){
							//override globals parameters value if block contains 'parameters' key
							angular.forEach($scope.report_experiments[experiment_id].declaration.blocks[key_block]['parameters'], function(value_block_parameter, key_block_parameter){
								if($scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter] == undefined){
									$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter] = [];
								}
								$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter].push(value_block_parameter);

								$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter] = unique($scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter]);
							});
						}
						else{
							//get globals parameters value if block doesn't contain 'parameters' key
							angular.forEach($scope.report_experiments[experiment_id].declaration.globals[key_algorithm], function(value_block_parameter, key_block_parameter){
								if($scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter] == undefined){
									$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter] = [];
								}
								$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter].push(value_block_parameter);
								$scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter] = unique($scope.report_algorithm_parameters_experiment[experiment_id][key_algorithm][key_block_parameter]);
							});
						}
					}
				});
			}
		});
	}

	function setExperimentAlias(experiment_id){
		$scope.report_experiments_alias[experiment_id] = experiment_id.split("/").pop();
		$scope.report_experiments_alias_from_content[experiment_id] = experiment_id.split("/").pop();
	}

	function setExperimentAliasFromContent(experiment_id, experiment_alias){
		$scope.report_experiments_alias[experiment_id] = experiment_alias;
		$scope.report_experiments_alias_from_content[experiment_id] = experiment_alias;
	}

	function unique(list){
		let result = [];
		$.each(list, function(i, e) {
			if ($.inArray(e, result) == -1) result.push(e);
		});
		return result;
	}
}]);
