/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

//Directive used to export graph/tables on click
angular.module('reportApp').directive("buttonexportitem", function(){
	return {
		link:function(scope, element, attrs){
			element.bind("click", function(){
				let the_element = element.context.parentElement.parentElement.parentElement.parentElement.parentElement.parentElement;
				if(the_element.id.indexOf("chart_") == 0){
					let content_detail = {};
					let chart_id = the_element.id;

					content_detail["name"] = scope.plots_details[chart_id].data.output[0];

					if(scope.plots_details[chart_id].data.plotter != undefined){
						content_detail["selected_plotter"] = scope.plots_details[chart_id].data.plotter;
					}
					if(scope.plots_details[chart_id].data.parameter != undefined){
						content_detail["selected_template"] = scope.plots_details[chart_id].data.parameter;
					}
					if(scope.plots_details[chart_id].data.merged != undefined){
						content_detail["merged"] = scope.plots_details[chart_id].data.merged;
					}

					let accessnumber = "no_number";
					if(scope.report.number == scope.report_number){
						accessnumber = "number";
					}
					_get_chart(scope, content_detail, element[0].id, chart_id);
				}
				else if(the_element.id.indexOf("table_") == 0){
					let separator = ",";
					let csv_text = export_table_as_csv(scope, the_element.id, separator);
					let output_filename = the_element.id + ".csv";
					download(csv_text, output_filename, "text/plain");
				}
			});
		}
	};

	function export_table_as_csv(scope, table_id, separator){
		let report_experiments = scope.$parent.report_experiments;
		let report_experiments_alias = scope.$parent.report_experiments_alias;
		let floating_point_precision = scope.$parent.floating_point_precision;
		let report = scope.report;
		let final_table = [];
		let table_headers = [];
		angular.forEach(scope.tables_details[table_id], function(table_value, table_key){
			if(table_value.selected){
				table_headers.push(table_value.name);
			}
		});

		final_table.push(table_headers);

		angular.forEach(report_experiments, function(experiment_value, experiment_key){
			let table_items = [];

			angular.forEach(table_headers, function(table_value, table_key){
				let experiment_name = experiment_key;
				let analyzer_block = report_experiments[experiment_name].analyzer_block;
				let report_algorithm_parameters_experiment = scope.$parent.report_algorithm_parameters_experiment;

				if(table_value != "experiment"){
					if(analyzer_block == undefined){
						angular.forEach(report_experiments, function(value, key){
							angular.forEach(value.declaration.analyzers, function(value_analyzer, key_analyzer){
								if(value_analyzer.algorithm == report.analyzer && (key_analyzer in report_experiments[experiment_name].declaration.analyzers)){
									report_experiments[experiment_name].analyzer_block = key_analyzer;
									analyzer_block = report_experiments[experiment_name].analyzer_block;
								}
							});
						});
					}

					if(table_value.indexOf("execution_time.") == 0){
						//execution time information
						let block_name = table_value.split("execution_time.")[1].split("[s]")[0];
						if(Object.keys(report_experiments[experiment_name].execution_info).length === 0){
							table_items.push("-");
						}
						else{
							if(report_experiments[experiment_name].execution_info[block_name] == undefined){
								table_items.push("-");
							}
							else{
								table_items.push((report_experiments[experiment_name].execution_info[block_name].linear_execution_time).toFixed(floating_point_precision.selected));
							}
						}
					}
					else if(table_value.indexOf("experiment.") == 0){
						//total execution time information
						let block_name = table_value.split("execution_time.")[1];
						let total_time = 0;
						if(Object.keys(report_experiments[experiment_name].execution_info).length === 0){
							total_time = "-";
						}
						else{
							angular.forEach(report_experiments[experiment_name].execution_info, function(value, key){
								total_time += value.linear_execution_time;
							});
						}

						table_items.push(total_time.toFixed(floating_point_precision.selected));
					}
					else if(table_value.indexOf("algorithm_parameter.") == 0){
						//total execution time information
						let block_name = table_value.split("algorithm_parameter.")[1];
						let algorithm_name = block_name.split("__")[0];
						let parameter_name = block_name.split("__")[1];
						if(report_algorithm_parameters_experiment[experiment_name][algorithm_name] != undefined){
							let value = "";
							for(let i = 0; i < report_algorithm_parameters_experiment[experiment_name][algorithm_name][parameter_name].length; i++){
								if(i > 0 && i < report_algorithm_parameters_experiment[experiment_name][algorithm_name][parameter_name].length -1 ){
									value += ",";
								}
								value = report_algorithm_parameters_experiment[experiment_name][algorithm_name][parameter_name][i];
							}

							table_items.push(value);
						}
						else{
							table_items.push("-");
						}
					}
					else{
						//results information
						table_items.push((report_experiments[experiment_name].results[analyzer_block][table_value].value).toFixed(floating_point_precision.selected));
					}
				}
				else{
					let experiment_alias = report_experiments_alias[experiment_name];
					table_items.push(experiment_alias);
				}

			});

			final_table.push(table_items);

		});

		let csv_text = "";
		for(let i = 0; i < final_table.length; i++){
			if(i != 0){
				csv_text += "\n";
			}

			for(let j = 0; j < final_table[i].length; j++){
				if( j != 0){
					csv_text += separator;
				}
				csv_text += final_table[i][j];
			}
		}

		let pre_base64_text = "data:text/plain;base64,";
		let base_64_csv_text = pre_base64_text + Base64.encode(csv_text);
		return base_64_csv_text;
	}

	//Retrieve chart from api and display on proper item
	function _get_chart(scope, sub_content, content_type, chart_id){
		let required_plotter = [];
		let set_plotter = '';
		if(sub_content.selected_plotter != undefined){
			//required_plotter.push(sub_content.selected_plotter);
			set_plotter = sub_content.selected_plotter;
			let requested_dataformat = '';
			//Get other plotters for required dataformat
			for (let i = 0; i < scope.report.plotters.length; i++){
				if(scope.report.plotters[i].name == sub_content.selected_plotter){
					requested_dataformat = scope.report.plotters[i].dataformat;
					break;
				}
			}

			//Get default plotter for required dataformat
			for (let i = 0; i < scope.report.defaultplotters.length; i++){
				if(scope.report.defaultplotters[i].dataformat == requested_dataformat){
					sub_content.defaultplotter = scope.report.defaultplotters[i];
					break;
				}
			}

			required_plotter.push(sub_content.defaultplotter.plotter);

			//Get other plotters for required dataformat
			for (let i = 0; i < scope.report.plotters.length; i++){
				if(scope.report.plotters[i].dataformat == requested_dataformat && scope.report.plotters[i].name != sub_content.defaultplotter.plotter){
					required_plotter.push(scope.report.plotters[i].name);
				}
			}
		}
		else{
			//Get default plotter for required dataformat
			for (let i = 0; i < scope.report.defaultplotters.length; i++){
				if(scope.report.defaultplotters[i].dataformat == sub_content.description){
					sub_content.defaultplotter = scope.report.defaultplotters[i];
					break;
				}
			}

			required_plotter.push(sub_content.defaultplotter.plotter);
			set_plotter = sub_content.defaultplotter.plotter;

			//Get other plotters for required dataformat
			for (let i = 0; i < scope.report.plotters.length; i++){
				if(scope.report.plotters[i].dataformat == sub_content.description && scope.report.plotters[i].name != sub_content.defaultplotter.plotter){
					required_plotter.push(scope.report.plotters[i].name);
				}
			}
		}

		let plotterparameter = [];
		//Get other plotterparameter
		for (let i = 0; i < scope.report.plotterparameter.length; i++){
			plotterparameter.push(scope.report.plotterparameter[i].name);
		}

		let chart_name = sub_content.name;

		let legend_experiments = '';
		for(let i = 0; i < scope.report.experiments.length; i++){
			if(i == 0)
				legend_experiments = scope.report_experiments_alias[scope.report.experiments[i]];
			else
				legend_experiments = legend_experiments+ "&" +scope.report_experiments_alias[scope.report.experiments[i]];
		}

		let request_data = {
			experiment:   scope.report.experiments,
			analyzer:     [scope.report.analyzer],
			output:       [chart_name],
			plotter:      set_plotter,
			legend:       legend_experiments,
			//height:       300,
			//width:        400,
		};

		base_url = scope.report.url_prefix;

		let plot_detail = {};
		plot_detail["required_plotter"] = required_plotter;
		plot_detail["data"] = request_data;
		if(sub_content.selected_template != undefined){// && sub_content.selected_template != "Default")
			plot_detail["selected_template"] = sub_content.selected_template;
			request_data.parameter = plot_detail["selected_template"];
		}
		else{
			plot_detail["selected_template"] = sub_content.defaultplotter.parameter;
			request_data.parameter = plot_detail["selected_template"];
		}

		if(sub_content.merged != undefined){// && sub_content.selected_template != "Default")
			plot_detail["merged"] = sub_content.merged;
			request_data.merged = plot_detail["merged"];
		}
		else{
			plot_detail["merged"] = true;
			request_data.merged = plot_detail["merged"];
		}

		beat.experiments.utils.getPlotData(base_url,
			request_data, required_plotter, plotterparameter, content_type,
			function(r_image_data, selected_content_type) {
				download(r_image_data, chart_id + "." + content_type.toLowerCase(), selected_content_type);
			}
		);
	}


});

